/*
 * Copyright (c) 2018 elementary LLC. (https://github.com/elementary/vala-lint)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301 USA.
 */

public class ValaLint.Checks.NoSpaceCheck : Check {
    public NoSpaceCheck () {
        Object (
            title: "no-space",
            description: _("Checks for missing spaces")
        );

        state = Config.get_state (title);
    }

    public override void check (Vala.ArrayList<ParseResult?> parse_result,
                                ref Vala.ArrayList<FormatMistake?> mistake_list) {

    }

    public void check_list (Vala.List<Vala.CodeNode?> list,
                            ref Vala.ArrayList<FormatMistake?> mistake_list) {
        if (state == Config.State.OFF) {
            return;
        }

        if (list.size > 1) {
            for (int i = 0; i < list.size - 1; i++) {
                var expr = list[i];
                var reference = expr.source_reference.end;

                if (expr is Vala.Parameter) {
                    var parameter = expr as Vala.Parameter;
                    if (parameter.initializer != null) {
                        reference = parameter.initializer.source_reference.end;
                    }
                }

                // Move char to comma seperator
                int offset = 0;
                while (reference.pos[offset] != ',') {
                    offset += 1;
                }
                offset += 1;

                if (reference.pos[offset] != ' ' && reference.pos[offset] != '\n') {
                    var begin = Utils.shift_location (reference, offset + 1);
                    var end = Utils.shift_location (begin, 1);

                    add_mistake ({ this, begin, end, _("Expected a whitespace in between") }, ref mistake_list);
                }
            }
        }
    }

    public void check_binary_expression (Vala.BinaryExpression expr,
                                         ref Vala.ArrayList<FormatMistake?> mistake_list) {
        char* char_before = expr.left.source_reference.end.pos;

        if (char_before[0] != ' ' && char_before[0] != '\n' && char_before[0] != ')') {
            var begin = Utils.shift_location (expr.left.source_reference.end, 1);
            var end = Utils.shift_location (begin, 1);

            add_mistake ({ this, begin, end, _("Expected spaces around operators") }, ref mistake_list);
        }

        char* char_after = expr.right.source_reference.begin.pos - 1;
        if (char_after[0] != ' ' && char_after[0] != '\n' && char_after[0] != '(' && char_after[1] != '(') {
            var begin = expr.right.source_reference.begin;
            var end = Utils.shift_location (begin, 1);

            add_mistake ({ this, begin, end, _("Expected spaces around operators") }, ref mistake_list);
        }
    }
}
